/*
 * Copyright 2012-2016 Red Hat, Inc.
 *
 * This file is part of Thermostat.
 *
 * Thermostat is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2, or (at your
 * option) any later version.
 *
 * Thermostat is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Thermostat; see the file COPYING.  If not see
 * <http://www.gnu.org/licenses/>.
 *
 * Linking this code with other modules is making a combined work
 * based on this code.  Thus, the terms and conditions of the GNU
 * General Public License cover the whole combination.
 *
 * As a special exception, the copyright holders of this code give
 * you permission to link this code with independent modules to
 * produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also
 * meet, for each linked independent module, the terms and conditions
 * of the license of that module.  An independent module is a module
 * which is not derived from or based on this code.  If you modify
 * this code, you may extend this exception to your version of the
 * library, but you are not obligated to do so.  If you do not wish
 * to do so, delete this exception statement from your version.
 */

package com.redhat.thermostat.notes.client.cli.internal;

import com.redhat.thermostat.common.cli.CompleterService;
import com.redhat.thermostat.testutils.StubBundleContext;
import org.junit.Test;

import static com.redhat.thermostat.testutils.Asserts.assertCommandIsNotRegistered;
import static com.redhat.thermostat.testutils.Asserts.assertCommandIsRegistered;
import static com.redhat.thermostat.testutils.Asserts.assertServiceIsRegistered;
import static org.junit.Assert.assertEquals;

public class ActivatorTest {

    @Test
    public void testCommandsAreRegistered() throws Exception {
        StubBundleContext context = new StubBundleContext();
        Activator activator = new Activator();

        activator.start(context);

        assertCommandIsRegistered(context, AddNoteCommand.NAME, AddNoteCommand.class);
        assertCommandIsRegistered(context, DeleteNoteCommand.NAME, DeleteNoteCommand.class);
        assertCommandIsRegistered(context, UpdateNoteCommand.NAME, UpdateNoteCommand.class);
        assertCommandIsRegistered(context, ListNotesCommand.NAME, ListNotesCommand.class);
        assertServiceIsRegistered(context, CompleterService.class, NoteIdCompleterService.class);

        assertEquals(5, context.getAllServices().size());

        activator.stop(context);

        assertEquals(1, context.getAllServices().size());
        assertCommandIsNotRegistered(context, AddNoteCommand.NAME, AddNoteCommand.class);
        assertCommandIsNotRegistered(context, DeleteNoteCommand.NAME, DeleteNoteCommand.class);
        assertCommandIsNotRegistered(context, UpdateNoteCommand.NAME, UpdateNoteCommand.class);
        assertCommandIsNotRegistered(context, ListNotesCommand.NAME, ListNotesCommand.class);
        assertServiceIsRegistered(context, CompleterService.class, NoteIdCompleterService.class);
    }

}
